@@ -1,6 +1,7 @@
 import re
 import copy
 import logging
+import traceback
 from pathlib import Path
 from textwrap import dedent
 from datetime import datetime
@@ -101,8 +102,6 @@ class CodeActAgent:
 
         self.logger = logging.getLogger(self.__class__.__name__ + ":" + self.name)
         self.logger.setLevel(self.verbosity_level)
-        if not self.logger.hasHandlers():
-            self.logger.addHandler(logging.StreamHandler())
 
     def get_all_agents(self) -> List[Self]:
         agents = [self]
@@ -163,7 +162,11 @@ class CodeActAgent:
                         f"Planning step {step_number} started", run_id=run_id, session_id=session_id
                     )
                     new_messages = await self._run_planning_step(
-                        messages, tools, session_id, event_bus
+                        messages=messages,
+                        tools=tools,
+                        run_id=run_id,
+                        session_id=session_id,
+                        event_bus=event_bus,
                     )
                     messages.extend(new_messages)
                     self._log(
@@ -243,18 +246,29 @@ class CodeActAgent:
         self._log(
             f"Step inputs: {messages}", run_id=run_id, session_id=session_id, level=logging.DEBUG
         )
-        output_stream = self.llm.astream(messages, stop=self.prompts.stop_sequences)
-
+        self._log("LLM generates outputs...", run_id=run_id, session_id=session_id)
         output_text = ""
-        async for event in output_stream:
-            delta = event.choices[0].delta
-            if isinstance(delta.content, str):
-                chunk = delta.content
-            elif isinstance(delta.content, list):
-                chunk = "\n".join([str(item) for item in delta.content])
-            output_text += chunk
-            await self._publish_event(event_bus, session_id, EventType.OUTPUT, chunk)
-        await self._publish_event(event_bus, session_id, EventType.OUTPUT, "\n")
+        try:
+            output_stream = self.llm.astream(messages, stop=self.prompts.stop_sequences)
+            async for event in output_stream:
+                delta = event.choices[0].delta
+                if isinstance(delta.content, str):
+                    chunk = delta.content
+                elif isinstance(delta.content, list):
+                    chunk = "\n".join([str(item) for item in delta.content])
+                output_text += chunk
+                await self._publish_event(event_bus, session_id, EventType.OUTPUT, chunk)
+                if any(
+                    stop_sequence in output_text for stop_sequence in self.prompts.stop_sequences
+                ):
+                    break
+            await self._publish_event(event_bus, session_id, EventType.OUTPUT, "\n")
+
+        except Exception:
+            exception = traceback.format_exc()
+            error_text = f"LLM failed with error: {exception}. Please try again."
+            self._log(error_text, run_id=run_id, session_id=session_id, level=logging.ERROR)
+            return []
 
         if (
             output_text
@@ -264,20 +278,26 @@ class CodeActAgent:
             chunk = self.prompts.end_code_sequence + "\n"
             output_text += chunk
 
-        self._log(
-            f"Step output: {output_text}", run_id=run_id, session_id=session_id, level=logging.DEBUG
-        )
-
         for stop_sequence in self.prompts.stop_sequences:
             if stop_sequence in output_text:
                 output_text = output_text.split(stop_sequence)[0].strip()
                 break
 
+        self._log(
+            f"Step output: {output_text}", run_id=run_id, session_id=session_id, level=logging.DEBUG
+        )
+        self._log("LLM generated outputs!", run_id=run_id, session_id=session_id)
+
+        # Code detection
         code_action = extract_code_from_text(output_text)
+
+        # No code action
         new_messages = []
         if code_action is None:
+            self._log("No tool calls detected", run_id=run_id, session_id=session_id)
             new_messages.append(ChatMessage(role="assistant", content=output_text))
             if self._is_final_answer(output_text):
+                self._log("Final answer found!", run_id=run_id, session_id=session_id)
                 return new_messages
             assert self.prompts.no_code_action is not None
             no_code_action_prompt = self.prompts.no_code_action.render()
@@ -297,11 +317,11 @@ class CodeActAgent:
             role="assistant",
             content=output_text,
         )
-        await self._publish_event(event_bus, session_id, EventType.TOOL_CALL, code_action)
         new_messages.append(tool_call_message)
 
         # Execute code
         try:
+            self._log("Executing code...", run_id=run_id, session_id=session_id)
             code_result = await python_executor.ainvoke(code_action)
             self._log(
                 f"Code result: {code_result}",
@@ -314,6 +334,7 @@ class CodeActAgent:
             new_messages.append(code_result_message)
             tool_output: str = str(code_result_message.content[0]["text"]) + "\n"
             await self._publish_event(event_bus, session_id, EventType.TOOL_RESPONSE, tool_output)
+            self._log("Code was executed!", run_id=run_id, session_id=session_id)
         except Exception as e:
             new_messages.append(ChatMessage(role="user", content=f"Error: {e}"))
             self._log(f"Code error: {e}", run_id=run_id, session_id=session_id, level=logging.DEBUG)
@@ -369,6 +390,7 @@ class CodeActAgent:
         self,
         messages: ChatMessages,
         tools: List[Tool],
+        run_id: str,
         session_id: str,
         event_bus: AgentEventBus | None = None,
     ) -> ChatMessages:
@@ -387,26 +409,33 @@ class CodeActAgent:
         )
         input_messages = [ChatMessage(role="user", content=planning_prompt)]
 
-        output_stream = self.llm.astream(input_messages, stop=[self.prompts.end_plan_sequence])
+        try:
+            output_stream = self.llm.astream(input_messages, stop=[self.prompts.end_plan_sequence])
 
-        plan_prefix = self.prompts.plan_prefix.render().strip() + "\n\n"
-        await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_prefix)
-        output_text = plan_prefix
+            plan_prefix = self.prompts.plan_prefix.render().strip() + "\n\n"
+            await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_prefix)
+            output_text = plan_prefix
 
-        async for event in output_stream:
-            delta = event.choices[0].delta
-            chunk = delta.content
-            if chunk is None:
-                continue
-            assert isinstance(chunk, str), event
-            output_text += chunk
-            await self._publish_event(event_bus, session_id, EventType.OUTPUT, chunk)
+            async for event in output_stream:
+                delta = event.choices[0].delta
+                chunk = delta.content
+                if chunk is None:
+                    continue
+                assert isinstance(chunk, str), event
+                output_text += chunk
+                await self._publish_event(event_bus, session_id, EventType.OUTPUT, chunk)
 
-        plan_suffix = "\n\n" + self.prompts.plan_suffix.render().strip() + "\n\n"
-        await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_suffix)
-        output_text += plan_suffix
+            plan_suffix = "\n\n" + self.prompts.plan_suffix.render().strip() + "\n\n"
+            await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_suffix)
+            output_text += plan_suffix
 
-        return [ChatMessage(role="assistant", content=output_text)]
+            return [ChatMessage(role="assistant", content=output_text)]
+
+        except Exception:
+            exception = traceback.format_exc()
+            error_text = f"LLM failed with error: {exception}. Please try again."
+            self._log(error_text, run_id=run_id, session_id=session_id, level=logging.ERROR)
+            return []
 
     async def _publish_event(
         self,
