@@ -1,10 +1,9 @@
 import os
 import copy
 import logging
-import traceback
 from typing import Dict, Any, List, cast, AsyncGenerator, Optional
 
-import tiktoken
+from tiktoken import encoding_for_model
 from dotenv import load_dotenv
 from pydantic import BaseModel
 from openai import AsyncOpenAI, AsyncStream
@@ -17,6 +16,10 @@ BASE_URL = os.getenv("BASE_URL", "https://openrouter.ai/api/v1")
 API_KEY = os.getenv("OPENROUTER_API_KEY", "")
 HTTP_REFERRER = os.getenv("HTTP_REFERRER", "https://github.com/IlyaGusev/codearkt/")
 X_TITLE = os.getenv("X_TITLE", "CodeArkt")
+HEADERS = {
+    "HTTP-Referer": HTTP_REFERRER,
+    "X-Title": X_TITLE,
+}
 
 
 class FunctionCall(BaseModel):  # type: ignore
@@ -46,7 +49,7 @@ ChatMessages = List[ChatMessage]
 
 
 def count_openai_tokens(messages: ChatMessages) -> int:
-    encoding = tiktoken.encoding_for_model("gpt-4o")
+    encoding = encoding_for_model("gpt-4o")
     tokens = [encoding.encode(str(message.content)) for message in messages]
     return sum(len(token) for token in tokens) + 3 * len(messages)
 
@@ -103,26 +106,18 @@ class LLM:
             for message in messages
         ]
 
-        async with AsyncOpenAI(base_url=self._base_url, api_key=self._api_key) as api:
-            for retry_num in range(self._num_retries):
-                try:
-                    stream: AsyncStream[ChatCompletionChunk] = await api.chat.completions.create(
-                        model=self._model_name,
-                        messages=casted_messages,
-                        stream=True,
-                        extra_headers={
-                            "HTTP-Referer": HTTP_REFERRER,
-                            "X-Title": X_TITLE,
-                        },
-                        **api_params,
-                    )
-                    async for event in stream:
-                        event_typed: ChatCompletionChunk = event
-                        yield event_typed
-                    break
-                except Exception:
-                    traceback.print_exc()
-                    if retry_num == self._num_retries - 1:
-                        raise
-                    self._logger.warning("Retrying LLM call...")
-                    continue
+        async with AsyncOpenAI(
+            base_url=self._base_url,
+            api_key=self._api_key,
+            max_retries=self._num_retries,
+        ) as api:
+            stream: AsyncStream[ChatCompletionChunk] = await api.chat.completions.create(
+                model=self._model_name,
+                messages=casted_messages,
+                stream=True,
+                extra_headers=HEADERS,
+                **api_params,
+            )
+            async for event in stream:
+                event_typed: ChatCompletionChunk = event
+                yield event_typed
