@@ -1,25 +1,29 @@
 import textwrap
 import time
-import httpx
+import os
 import asyncio
 import atexit
-import signal
 import json
+import traceback
 from typing import Optional, Any, List, Dict, Sequence
 import threading
 
-import docker
+from docker import from_env as docker_from_env
 from docker.models.containers import Container
 from docker.client import DockerClient
+from docker.errors import DockerException, ImageNotFound, NotFound
 from docker.models.networks import Network
-from pydantic import BaseModel
+from pydantic import BaseModel, ValidationError
+from httpx import AsyncClient, HTTPError, TimeoutException, Limits, RequestError
 
 from codearkt.llm import ChatMessage
 from codearkt.tools import fetch_tools
 from codearkt.util import get_unique_id, truncate_content, is_correct_json
 
 
-IMAGE: str = "phoenix120/codearkt_http:v3"
+SHA_DIGEST: str = "sha256:109502152db0acbdea118ccd827aaa90cfb92aa53a0cb1d816845cf9b6721f74"
+DEFAULT_IMAGE: str = f"phoenix120/codearkt_http@{SHA_DIGEST}"
+IMAGE: str = os.getenv("CODEARKT_EXECUTOR_IMAGE", DEFAULT_IMAGE)
 MEM_LIMIT: str = "512m"
 CPU_QUOTA: int = 50000
 CPU_PERIOD: int = 100000
@@ -27,14 +31,13 @@ EXEC_TIMEOUT: int = 24 * 60 * 60  # 24 hours
 CLEANUP_TIMEOUT: int = 10
 PIDS_LIMIT: int = 64
 NET_NAME: str = "sandbox_net"
-CONTAINER_NAME: str = "codearkt_http"
 
 _CLIENT: Optional[DockerClient] = None
 _CONTAINER: Optional[Container] = None
 _DOCKER_LOCK: threading.Lock = threading.Lock()
 
 
-def cleanup_container(signum: Optional[Any] = None, frame: Optional[Any] = None) -> None:
+def cleanup_container() -> None:
     global _CONTAINER
 
     acquired: bool = _DOCKER_LOCK.acquire(timeout=CLEANUP_TIMEOUT)
@@ -44,19 +47,14 @@ def cleanup_container(signum: Optional[Any] = None, frame: Optional[Any] = None)
                 _CONTAINER.stop(timeout=CLEANUP_TIMEOUT)
                 _CONTAINER.remove(force=True)
                 _CONTAINER = None
-            except Exception:
+            except DockerException:
                 pass
     finally:
         if acquired:
             _DOCKER_LOCK.release()
 
-    if signum == signal.SIGINT:
-        raise KeyboardInterrupt()
-
 
 atexit.register(cleanup_container)
-signal.signal(signal.SIGINT, cleanup_container)
-signal.signal(signal.SIGTERM, cleanup_container)
 
 
 class ExecResult(BaseModel):  # type: ignore
@@ -66,8 +64,11 @@ class ExecResult(BaseModel):  # type: ignore
 
     def to_message(self) -> ChatMessage:
         image_content: List[Dict[str, Any]] | None = None
-        output: str = "Stdout:\n" + self.stdout + "\n\n"
-        if self.result is not None:
+        output: str = ""
+        if self.stdout:
+            output += "Output:\n" + self.stdout + "\n\n"
+
+        if self.result:
             try:
                 json_result = json.loads(str(self.result))
                 if isinstance(json_result, dict) and "image_base64" in json_result:
@@ -82,8 +83,10 @@ class ExecResult(BaseModel):  # type: ignore
                 pass
             if not image_content:
                 output += "Last expression:\n" + str(self.result) + "\n\n"
+
         if self.error:
             output += "Error: " + self.error
+
         output = output.strip()
 
         content = []
@@ -94,25 +97,25 @@ class ExecResult(BaseModel):  # type: ignore
         return ChatMessage(role="user", content=content)
 
 
-def init_docker() -> docker.DockerClient:
-    client = docker.from_env()
+def init_docker() -> DockerClient:
+    client = docker_from_env()
     try:
         client.ping()  # type: ignore
-    except docker.errors.DockerException as exc:
+    except DockerException as exc:
         raise RuntimeError(
             "Docker daemon is not running or not accessible – skipping PythonExecutor setup."
         ) from exc
 
     try:
         client.images.get(IMAGE)
-    except docker.errors.ImageNotFound:
+    except ImageNotFound:
         try:
             client.images.pull(IMAGE)
-        except docker.errors.DockerException as exc:
+        except DockerException as exc:
             raise RuntimeError(
                 f"Docker image '{IMAGE}' not found locally and failed to pull automatically."
             ) from exc
-    except docker.errors.DockerException as exc:
+    except DockerException as exc:
         raise RuntimeError("Failed to query Docker images – ensure Docker is available.") from exc
     return client
 
@@ -120,7 +123,7 @@ def init_docker() -> docker.DockerClient:
 def run_network(client: DockerClient) -> Network:
     try:
         net = client.networks.get(NET_NAME)
-    except docker.errors.NotFound:
+    except NotFound:
         net = client.networks.create(
             NET_NAME,
             driver="bridge",
@@ -135,7 +138,6 @@ def run_network(client: DockerClient) -> Network:
 def run_container(client: DockerClient, net_name: str) -> Container:
     return client.containers.run(
         IMAGE,
-        name=CONTAINER_NAME,
         detach=True,
         auto_remove=True,
         ports={"8000/tcp": None},
@@ -171,11 +173,8 @@ class PythonExecutor:
             client = _CLIENT
 
             if not _CONTAINER:
-                try:
-                    _CONTAINER = client.containers.get(CONTAINER_NAME)
-                except docker.errors.NotFound:
-                    net = run_network(client)
-                    _CONTAINER = run_container(client, str(net.name))
+                net = run_network(client)
+                _CONTAINER = run_container(client, str(net.name))
 
             self.container = _CONTAINER
         self.tools_server_host = tools_server_host
@@ -226,11 +225,14 @@ class PythonExecutor:
             "interpreter_id": self.interpreter_id,
         }
 
-        async with httpx.AsyncClient(limits=httpx.Limits(keepalive_expiry=0)) as client:
-            resp = await client.post(f"{self.url}/exec", json=payload, timeout=EXEC_TIMEOUT)
-            resp.raise_for_status()
-            out = resp.json()
-            result: ExecResult = ExecResult.model_validate(out)
+        try:
+            async with AsyncClient(limits=Limits(keepalive_expiry=0)) as client:
+                resp = await client.post(f"{self.url}/exec", json=payload, timeout=EXEC_TIMEOUT)
+                resp.raise_for_status()
+                out = resp.json()
+                result: ExecResult = ExecResult.model_validate(out)
+        except (HTTPError, TimeoutException, ValueError, ValidationError):
+            result = ExecResult(stdout="", error=traceback.format_exc())
 
         if result.stdout:
             result.stdout = truncate_content(result.stdout)
@@ -253,22 +255,25 @@ class PythonExecutor:
             "interpreter_id": self.interpreter_id,
         }
         try:
-            async with httpx.AsyncClient(limits=httpx.Limits(keepalive_expiry=0)) as client:
+            async with AsyncClient(limits=Limits(keepalive_expiry=0)) as client:
                 response = await client.post(
                     f"{self.url}/cleanup", json=payload, timeout=CLEANUP_TIMEOUT
                 )
                 response.raise_for_status()
-        except Exception:
+        except (HTTPError, TimeoutException):
             pass
 
     async def _wait_for_ready(self, max_wait: int = 60) -> None:
+        delay = 0.1
         start_time = time.time()
         while time.time() - start_time < max_wait:
             try:
                 output = await self._call_exec("print('ready')", send_tools=False)
-                assert output.stdout.strip() == "ready"
-                return
-            except (httpx.RequestError, httpx.TimeoutException, AssertionError):
+                if output.stdout.strip() == "ready":
+                    return
+            except (RequestError, TimeoutException, AssertionError):
                 pass
-            await asyncio.sleep(0.1)
+
+            await asyncio.sleep(delay)
+            delay = min(delay * 2, 3.0)
         raise RuntimeError("Container failed to become ready within timeout")
